<?php

namespace Shopping\Payment\Providers;

use Illuminate\Support\ServiceProvider;

class PaymentServiceProvider extends ServiceProvider
{
    /**
     * Indicates if loading of the provider is deferred.
     *
     * @var bool
     */
    protected $defer = false;

    /**
     * Bootstrap the application events.
     *
     * @return void
     */
    public function boot()
    {
        // Load view
        $this->loadViewsFrom(__DIR__ . '/../../resources/views', 'payment');

        // Load translation
        $this->loadTranslationsFrom(__DIR__ . '/../../resources/lang', 'payment');

        // Load migrations
        $this->loadMigrationsFrom(__DIR__ . '/../../database/migrations');

        // Call pblish redources function
        $this->publishResources();

    }

    /**
     * Register the service provider.
     *
     * @return void
     */
    public function register()
    {
        // Bind facade
        $this->app->bind('shopping.payment', function ($app) {
            return $this->app->make('Shopping\Payment\Payment');
        });

        // Bind Payment to repository
        $this->app->bind(
            'Shopping\Payment\Interfaces\PaymentRepositoryInterface',
            \Shopping\Payment\Repositories\Eloquent\PaymentRepository::class
        );        // Bind Transaction to repository
        $this->app->bind(
            'Shopping\Payment\Interfaces\TransactionRepositoryInterface',
            \Shopping\Payment\Repositories\Eloquent\TransactionRepository::class
        );

        $this->app->register(\Shopping\Payment\Providers\AuthServiceProvider::class);
                $this->app->register(\Shopping\Payment\Providers\EventServiceProvider::class);
        
        $this->app->register(\Shopping\Payment\Providers\RouteServiceProvider::class);
                
    }

    /**
     * Get the services provided by the provider.
     *
     * @return array
     */
    public function provides()
    {
        return ['shopping.payment'];
    }

    /**
     * Publish resources.
     *
     * @return void
     */
    private function publishResources()
    {
        // Publish configuration file
        $this->publishes([__DIR__ . '/../../config/config.php' => config_path('shopping/payment.php')], 'config');

        // Publish admin view
        $this->publishes([__DIR__ . '/../../resources/views' => base_path('resources/views/vendor/payment')], 'view');

        // Publish language files
        $this->publishes([__DIR__ . '/../../resources/lang' => base_path('resources/lang/vendor/payment')], 'lang');

        // Publish public files and assets.
        $this->publishes([__DIR__ . '/public/' => public_path('/')], 'public');
    }
}
